@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2012	PgPool Global Development Group
+ * Copyright (c) 2003-2016	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -58,7 +58,6 @@
 #include "pool_process_reporting.h"
 
 #define MAX_FILE_LINE_LEN    512
-#define MAX_USER_PASSWD_LEN  128
 
 extern void pcp_set_timeout(long sec);
 volatile sig_atomic_t pcp_exit_request; /* non 0 means SIGTERM(smart shutdown) or SIGINT(fast shutdown) has arrived */
@@ -178,6 +177,13 @@ pcp_do_child(int unix_fd, int inet_fd, char *pcp_conf_file)
 		}
 
 		rsize = ntohl(rsize);
+
+		if (rsize <= 0 || rsize >= MAX_PCP_PACKET_LENGTH)
+		{
+			pool_error("pcp_child: incorrect packet length (%d)", rsize);
+			exit(1);
+		}
+
 		if ((rsize - sizeof(int)) > 0)
 		{
 			buf = (char *)malloc(rsize - sizeof(int));
